/* There's no support for double buffered circular mode for I2S in CubeMX
 * so we need to write our own */

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx_hal.h"
#include "hal_i2s.h"

#ifdef HAL_I2S_MODULE_ENABLED

static void
I2S_DMATxCplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent; 
  /* Call user Tx complete callback */
  HAL_I2S_TxCpltCallback( hi2s );
}

static void
I2S_DMATxM1Cplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent; 
  /* Call user Tx M1 complete callback */
  HAL_I2S_TxM1CpltCallback( hi2s );
}

static void
I2S_DMATxHalfCplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent;
  /* Call user Tx half complete callback */
  HAL_I2S_TxHalfCpltCallback( hi2s );
}

static void
I2S_DMARxCplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent;  
  /* Call user Rx complete callback */
  HAL_I2S_RxCpltCallback( hi2s );
}

static void
I2S_DMARxM1Cplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent;  
  /* Call user Rx M1 complete callback */
  HAL_I2S_RxM1CpltCallback( hi2s );
}

static void
I2S_DMARxHalfCplt( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent;  
  /* Call user Rx half complete callback */
  HAL_I2S_RxHalfCpltCallback( hi2s );
}

static void
I2S_DMAError( DMA_HandleTypeDef * hdma )
{
  I2S_HandleTypeDef *hi2s = ( I2S_HandleTypeDef * ) ( ( DMA_HandleTypeDef * ) hdma )->Parent; 
  /* Disable Rx and Tx DMA Request */
  CLEAR_BIT( hi2s->Instance->CR2, ( SPI_CR2_RXDMAEN | SPI_CR2_TXDMAEN ) );
  hi2s->TxXferCount = 0U;
  hi2s->RxXferCount = 0U;

  hi2s->State = HAL_I2S_STATE_READY;

  /* Set the error code and execute error callback */
  SET_BIT( hi2s->ErrorCode, HAL_I2S_ERROR_DMA );
  /* Call user error callback */
  HAL_I2S_ErrorCallback( hi2s );
}

/**
  * @brief  Transmit an amount of data in non-blocking mode with DMA
  * @param  hi2s pointer to a I2S_HandleTypeDef structure that contains
  *	    the configuration information for I2S module
  * @param  pData a 16-bit pointer to the Transmit data buffer.
  * @param  Size number of data sample to be sent:
  * @retval HAL status
  */
HAL_StatusTypeDef
HAL_I2S_Transmit_DblBuf_DMA( I2S_HandleTypeDef * hi2s, u16 * pData0, u16 * pData1,
    u16 Size )
{
  if( ( pData0 == NULL ) || ( pData1 == NULL ) || ( Size == 0U ) )
  {
    return HAL_ERROR;
  }

  /* Process Locked */
  __HAL_LOCK( hi2s );

  if( hi2s->State != HAL_I2S_STATE_READY )
  {
    __HAL_UNLOCK( hi2s );
    return HAL_BUSY;
  }

  /* Set state and reset error code */
  hi2s->State = HAL_I2S_STATE_BUSY_TX;
  hi2s->ErrorCode = HAL_I2S_ERROR_NONE;
  hi2s->pTxBuffPtr = pData0;

  /* 16 bits samples */
  hi2s->TxXferSize = Size; 
  hi2s->TxXferCount = Size;

  /* Set the I2S Tx DMA Half transfer complete callback */
  hi2s->hdmatx->XferHalfCpltCallback = I2S_DMATxHalfCplt;

  /* Set the I2S Tx DMA transfer complete callback */
  hi2s->hdmatx->XferCpltCallback = I2S_DMATxCplt;
  
  /* Set the I2S Tx M1 DMA transfer complete callback */
  hi2s->hdmatx->XferM1CpltCallback = I2S_DMATxM1Cplt;

  /* Set the DMA error callback */
  hi2s->hdmatx->XferErrorCallback = I2S_DMAError;

  if( HAL_OK != HAL_DMAEx_MultiBufferStart_IT( hi2s->hdmatx,
      ( u32 ) pData0, ( u32 ) & hi2s->Instance->DR,
      ( u32 ) pData1, hi2s->TxXferSize ) )
  {
    /* Update SPI error code */
    SET_BIT( hi2s->ErrorCode, HAL_I2S_ERROR_DMA );
    hi2s->State = HAL_I2S_STATE_READY;

    __HAL_UNLOCK( hi2s );
    return HAL_ERROR;
  }

  /* Check if the I2S is already enabled */
  if( HAL_IS_BIT_CLR( hi2s->Instance->I2SCFGR, SPI_I2SCFGR_I2SE ) )
  {
    /* Enable I2S peripheral */
    __HAL_I2S_ENABLE( hi2s );
  }

  /* Check if the I2S Tx request is already enabled */
  if( HAL_IS_BIT_CLR( hi2s->Instance->CR2, SPI_CR2_TXDMAEN ) )
  {
    /* Enable Tx DMA Request */
    SET_BIT( hi2s->Instance->CR2, SPI_CR2_TXDMAEN );
  }

  __HAL_UNLOCK( hi2s );
  return HAL_OK;
}

/**
  * @brief  Receive an amount of data in non-blocking mode with DMA
  * @param  hi2s pointer to a I2S_HandleTypeDef structure that contains
  *	    the configuration information for I2S module
  * @param  pData a 16-bit pointer to the Receive data buffer.
  * @param  Size number of data sample to be sent:
  * @retval HAL status
  */
HAL_StatusTypeDef
HAL_I2S_Receive_DblBuf_DMA( I2S_HandleTypeDef * hi2s, 
    u16 * pData0, u16 * pData1, u16 Size )
{
  if( ( pData0 == NULL ) || ( pData1 == NULL ) || ( Size == 0U ) )
  {
    return HAL_ERROR;
  }

  /* Process Locked */
  __HAL_LOCK( hi2s );

  if( hi2s->State != HAL_I2S_STATE_READY )
  {
    __HAL_UNLOCK( hi2s );
    return HAL_BUSY;
  }

  /* Set state and reset error code */
  hi2s->State = HAL_I2S_STATE_BUSY_RX;
  hi2s->ErrorCode = HAL_I2S_ERROR_NONE;
  hi2s->pRxBuffPtr = pData0;

  /* 24 bits samples */
  hi2s->RxXferSize = Size << 1U;
  hi2s->RxXferCount = Size << 1U;

  /* Set the I2S Rx DMA Half transfer complete callback */
  hi2s->hdmarx->XferHalfCpltCallback = I2S_DMARxHalfCplt;

  /* Set the I2S Rx DMA transfer complete callback */
  hi2s->hdmarx->XferCpltCallback = I2S_DMARxCplt;

  /* Set the I2S Rx M1 DMA transfer complete callback */
  hi2s->hdmarx->XferM1CpltCallback = I2S_DMARxM1Cplt;

  /* Set the DMA error callback */
  hi2s->hdmarx->XferErrorCallback = I2S_DMAError;

  /* Clear the Overrun Flag by a read operation to the SPI_DR register followed by a read access to the SPI_SR register. */
  __HAL_I2S_CLEAR_OVRFLAG( hi2s );

  // if( HAL_OK != HAL_DMA_Start_IT( hi2s->hdmarx,
  /* Enable the Rx DMA Stream/Channel */
  if( HAL_OK != HAL_DMAEx_MultiBufferStart_IT( hi2s->hdmarx,
      ( u32 ) & hi2s->Instance->DR, ( u32 ) pData0,
      ( u32 ) pData1, hi2s->RxXferSize ) )
  {
    /* Update SPI error code */
    SET_BIT( hi2s->ErrorCode, HAL_I2S_ERROR_DMA );
    hi2s->State = HAL_I2S_STATE_READY;

    __HAL_UNLOCK( hi2s );
    return HAL_ERROR;
  }

  /* Check if the I2S is already enabled */
  if( HAL_IS_BIT_CLR( hi2s->Instance->I2SCFGR, SPI_I2SCFGR_I2SE ) )
  {
    /* Enable I2S peripheral */
    __HAL_I2S_ENABLE( hi2s );
  }

  /* Check if the I2S Rx request is already enabled */
  if( HAL_IS_BIT_CLR( hi2s->Instance->CR2, SPI_CR2_RXDMAEN ) )
  {
    /* Enable Rx DMA Request */
    SET_BIT( hi2s->Instance->CR2, SPI_CR2_RXDMAEN );
  }

  __HAL_UNLOCK( hi2s );
  return HAL_OK;
}

#endif                          /* HAL_I2S_MODULE_ENABLED */
