// HD44780 driver.

#include "main.h"
#include "hd44780.h"
#include "cmsis_os.h"
#include "string.h"

bool lcd_state_on;
bool lcd_cursor_on;

static void
hd44780_set_dbus( u8 data )
{
  u32 port = LCD_D7_GPIO_Port->ODR & 0xfffffc3f;
  LCD_D7_GPIO_Port->ODR = port | data << 2;
}

static inline void
hd44780_set_e( int val )
{
  if( val )
    pinSet( LCD_E_GPIO_Port, LCD_E_Pin );
  else
    pinReset( LCD_E_GPIO_Port, LCD_E_Pin );
}

static inline void
hd44780_set_rs( int val )
{
  if( val )
    pinSet( LCD_RS_GPIO_Port, LCD_RS_Pin );
  else
    pinReset( LCD_RS_GPIO_Port, LCD_RS_Pin );
}

static void
strobe( void )
{
  hd44780_set_e( 1 );
  for( int i = 0; i < 28; i++ ) // 1us delay
    __asm( "nop" );
  hd44780_set_e( 0 );
  for( int i = 0; i < 28; i++ ) // 1us delay
    __asm( "nop" );
}

static void
wait_ready( unsigned ms )
{
  // Can't read data bus, wait.
  osDelay( ms );
}

// Send a command to the LCD in 4-bit or 8-bit mode.
void
hd44780_send_cmd( u8 cmd )
{
  hd44780_set_dbus( cmd & 0xf0 );
  hd44780_set_rs( 0 );          // set LCD to CMD mode
  strobe(  );                   // toggle E for LCD
  hd44780_set_dbus( ( cmd & 0xf ) << 4 );
  strobe(  );                   // toggle E for LCD
}

// Put a character to the LCD in 4-bit or 8-bit mode.
void
hd44780_putc( u8 u )
{
  hd44780_set_dbus( u & 0xf0 );
  hd44780_set_rs( 1 );          // set LCD to DATA mode
  strobe(  );                   // toggle E for LCD
  hd44780_set_dbus( ( u << 4 ) & 0xf0 );
  strobe(  );                   // toggle E for LCD
  wait_ready( 1 );
}

void
hd44780_set_cursor_position( u8 y, u8 x )
{
  if( y == 1 )
    y = 0x40;
  y += x;
  hd44780_send_cmd( 0x80 | y );
  wait_ready( 1 );
}

void
hd44780_set_direction( int reverse )
{
  hd44780_send_cmd( reverse ? ENTRY_DEC : ENTRY_INC );
  wait_ready( 1 );
}

void
hd44780_puts( u8 y, u8 x, const char *s, int len )
{
  hd44780_set_cursor_position( y, x );
  while( len-- )
    hd44780_putc( *s++ );
}

void
hd44780_center_string( u8 row, int offset, const char *s )
{
  u8 x = 0;
  int spaces;
  if( offset > 0 )
    x = offset;
  hd44780_set_cursor_position( row, x );
  spaces = 16 - ( offset < 0 ? -offset : offset ) - strlen( s );
  if( spaces < 0 )
    spaces = 0;
  spaces /= 2;
  while ( spaces-- )
  {
    hd44780_putc( ' ' );
    x++;
  }
  if( offset > 0 )
    offset = 0;
  while( *s && x < 16 + offset )
  {
    hd44780_putc( *s++ );
    x++;
  }
  while( x < 16 + offset )
  {
    hd44780_putc( ' ' );
    x++;
  }
}

// Initialize LCD in 4-bit mode.
void
hd44780_reset( void )
{
  // Output pins low and as output (except last pin).
  // If R/W is configured, this sets R/W to 0, i.e. write mode.
  hd44780_set_e( 0 );
  hd44780_set_rs( 0 );

  // Wait at least 15ms for the LCD to initialise.
  osDelay( 15 );

  // Reset as per p. 211 of HD44780 user guide.
  // Function set (8-bit data, two line mode, 5x7 dots).
  hd44780_set_dbus( 0x30 );
  strobe(  );
  osDelay( 10 );               // Need at least 4.1ms according to the guide.
  strobe(  );
  osDelay( 2 );                // Need at least 100us according to the guide.
  strobe(  );
  osDelay( 2 );                // Don't use BF here either.

  // move to 4-bit mode.
  // Function set with DL=0 sent in 8-bit mode to select a 4-bit interface.
  hd44780_set_dbus( 0x20 );
  strobe(  );
  osDelay( 6 );

  // Function set (4-bit data, two line mode, 5x7 dots).
  hd44780_send_cmd( 0x28 );
  osDelay( 6 );

  // Display off
  hd44780_send_cmd( DISP_OFF );
  osDelay( 5 );

  // Display clear
  hd44780_send_cmd( CLR_DISP );
  osDelay( 10 );

  // Entry mode, increasing cursor position.
  hd44780_send_cmd( ENTRY_INC );
  osDelay( 10 );

  // Display on.
  osDelay( 10 );
  hd44780_send_cmd( DISP_ON );
  wait_ready( 1 );
}

void
hd44780_clear( void )
{
  // Display clear
  hd44780_send_cmd( CLR_DISP );
  wait_ready( 10 );
}

void
hd44780_define_char( u8 address, const u8 * pattern )
{
  int           i;
  hd44780_send_cmd( 0x40 + ( address << 3 ) );
  wait_ready( 1 );
  for( i = 0; i < 8; ++i )
    hd44780_putc( *pattern++ );
}

void
hd44780_cursor_left( void )
{
  hd44780_send_cmd( 0x10 );
  wait_ready( 1 );
}

void
hd44780_cursor_right( void )
{
  hd44780_send_cmd( 0x14 );
  wait_ready( 1 );
}

void
hd44780_cursor_on( void )
{
  hd44780_send_cmd( 0x0e );
  wait_ready( 1 );
}

void
hd44780_display_on( void )
{
  hd44780_send_cmd( 0x0c );
  wait_ready( 1 );
}

void
hd44780_cursor_off( void )
{
  hd44780_send_cmd( 0x0c );
  wait_ready( 1 );
}

void
hd44780_display_off( void )
{
  hd44780_send_cmd( 0x08 );
  wait_ready( 1 );
}

