#include <stdlib.h>
#include <stdio.h>
#ifdef DEBUG_DFT
#include <debugio.h>
#endif                          // DEBUG_DFT
#include "main.h"
#include "keyer.h"
#include "morse.h"
#include "cmsis_os.h"
#include "threads.h"
#include "hal_i2s.h"
#include "hd44780.h"
#include "arm_math.h"
#include "configuration.h"

osThreadId    thread_tx_cw;
osThreadId    thread_tx_decode;
osThreadId    thread_rx_decode;
osThreadId    thread_timing_decode;
osThreadId    thread_lcd;
osThreadId    thread_options;
osThreadId    thread_init_lcd;
osThreadId    thread_beacon;

u8            cw_tx_buffer[CW_BUF_LEN];
u32           cw_wptr, cw_rptr;

static const u8 specials_chars[][8] = {
  {0, 0xe, 0x1f, 0x1f, 0x1f, 0xe, 0, 0},        // blob
  {0, 0xe, 0x11, 0x11, 0x11, 0xe, 0, 0},        // circle
  {0, 4, 0xe, 0x1f, 0xe, 4, 0, 0},      // star
  {0, 4, 0xa, 0x11, 0xa, 4, 0, 0},      // rhombus
  {4, 0xe, 0x15, 4, 4, 4, 4, 0},        // arrow up
  {4, 4, 4, 4, 0x15, 0xe, 4, 0},        // arrow down
  {0, 2, 0x6, 0xe, 0x6, 2, 0, 0},       // left
  {0, 8, 0xc, 0xe, 0xc, 8, 0, 0},       // right
  {0, 0, 4, 0xe, 0x1f, 0, 0, 0},        // up
  {0, 0, 0x1f, 0xe, 4, 0, 0, 0},        // down
  {0, 0, 8, 0x15, 0x15, 2, 0, 0},       // wave
};

void
init_lcd_thread( void const *argument )
{
  UNUSED( argument );
  int           i;

  TaskNotify( thread_options, NOTIFY_LOAD_OPTIONS );
  hd44780_reset(  );
  lcd_state_on = true;
  for( i = 0; i < 8; i++ )
    hd44780_define_char( i, specials_chars[i] );
  hd44780_center_string( 0, 0, "CW Keyer/Decoder" );
  hd44780_center_string( 1, 0, "version 0.1" );
  main_mode = M_VERSION;
  osThreadTerminate( thread_init_lcd );
}

void
options_thread( void const *argument )
{
  UNUSED( argument );
  u32           notify;
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( notify & NOTIFY_LOAD_OPTIONS )
    {
      load_NV_Options(  );
      compute_morse_timing(  );
      generate_sine_wave(  );
    }
    if( notify & NOTIFY_SAVE_OPTIONS )
    {
      save_NV_Options(  );
    }
    if( notify & NOTIFY_CHANGE_OPTIONS )
    {
      Change_Options( &Options, change_val );
    }
    if( notify & NOTIFY_SAVE_STRING )
    {
      save_current_string(  );
      TaskNotify( thread_lcd, NOTIFY_CLEAR );
    }
  }
}

#ifdef ENABLE_CW_DECODE
char          decoder_char_line[2][16];
volatile int  decoder_char_offset;      // unused for now
#endif                          // ENABLE_CW_DECODE

void
lcd_thread( void const *argument )
{
  UNUSED( argument );
  static enum main_modes current_mode = M_VERSION;
  int           i, offset, back_chars;
  char         *s;
  u8            ch;
  u32           notify;
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( ( notify & NOTIFY_CLEAR ) || current_mode != main_mode )
    {
      hd44780_clear(  );
      hd44780_cursor_off(  );
      lcd_cursor_on = false;
      current_mode = main_mode;
    }
    if( notify & NOTIFY_CHARS )
    {
#ifdef ENABLE_CW_DECODE
      if( Options.decoder_mode == DECODER_ONLY )
        continue;
#endif                          // ENABLE_CW_DECODE
      if( lcd_cursor_on )
      {
        hd44780_cursor_off(  );
        lcd_cursor_on = false;
      }
#ifdef ENABLE_CW_DECODE
      back_chars = ( Options.decoder_mode == DECODER_OFF ) ? 32 : 16;
#else
      back_chars = 32;
#endif                          // ENABLE_CW_DECODE
      hd44780_set_direction( 1 );
      for( i = 0; i < back_chars; i++ )
      {
        if( ( ch = cw_tx_buffer[( unsigned )
            ( cw_wptr - i - 1 ) % CW_BUF_LEN] ) != 0 )
        {
          if( ( i & 0xf ) == 0 )
            hd44780_set_cursor_position( i & 0x10 ? 0 : 1, 0xf );
          hd44780_putc( ch );
        }
      }
      hd44780_set_direction( 0 );
    }
#ifdef ENABLE_CW_DECODE
    if( notify & NOTIFY_DECODER_LINE )
    {
      if( Options.decoder_mode == DECODER_OFF || main_mode == M_ENC_CFG )
        continue;
      ch = *( decoder_char_buffer + CHAR_BUF_LEN - 1 + decoder_char_offset );
      i = ( Options.decoder_mode == DECODER_ONLY );     // line nr
      // compare all except last char which may change often
      s = decoder_char_buffer + CHAR_BUF_LEN - 16 + decoder_char_offset;
      if( memcmp( decoder_char_line[1], s, 15 ) )
      {
        memcpy( decoder_char_line[1], decoder_char_buffer +
            CHAR_BUF_LEN - 16 + decoder_char_offset, 16 );
        hd44780_puts( i, 0, decoder_char_line[1], 16 );
      }
      else if( decoder_char_line[1][15] != ch )
      {
        decoder_char_line[1][15] = ch;
        hd44780_set_cursor_position( i, 15 );
        hd44780_putc( decoder_char_line[1][15] );
      }
      if( Options.decoder_mode == DECODER_ONLY )
      {
        s = decoder_char_buffer + CHAR_BUF_LEN - 32 + decoder_char_offset;
        if( memcmp( decoder_char_line[0], s, 16 ) )
        {
          memcpy( decoder_char_line[0], s, 16 );
          hd44780_puts( 0, 0, decoder_char_line[0], 16 );
        }
      }
    }
#endif                          // ENABLE_CW_DECODE
    if( notify & NOTIFY_CURSOR_ON )
    {
#ifdef ENABLE_CW_DECODE
      back_chars = ( Options.decoder_mode == DECODER_OFF ) ? 32 : 16;
#else
      back_chars = 32;
#endif                          // ENABLE_CW_DECODE
      {
        i = cw_wptr - cw_rptr;
        if( i >= 0 && i < back_chars )
        {
          hd44780_set_cursor_position( i > 0xf ? 0 : 1, 0xf - ( i & 0xf ) );
        }
      }
      if( !lcd_cursor_on )
      {
        hd44780_cursor_on(  );
        lcd_cursor_on = true;
      }
    }
    if( notify & NOTIFY_CURSOR_OFF )
    {
      if( lcd_cursor_on )
      {
        hd44780_cursor_off(  );
        lcd_cursor_on = false;
      }
    }
    if( notify & NOTIFY_PARAMETER_DISPLAY )
    {
      display_parameter_value(  );
    }
    if( notify & NOTIFY_SAVEOPT_DISPLAY )
    {
      char          version[16];
      hd44780_center_string( 0, 0, "Saving Options" );
      sprintf( version, "version %d", Options.id );
      hd44780_center_string( 1, 0, version );
    }
    if( notify & NOTIFY_STRING_DISPLAY )
    {
      hd44780_clear(  );
      hd44780_set_cursor_position( 0, 0 );
      hd44780_putc( 'F' );
      hd44780_putc( '0' + fn_key / 10 );
      hd44780_putc( '0' + fn_key % 10 );
      hd44780_putc( ':' );
      offset = 0;
      i = string_wptr - 16;
      if( i > 12 )
      {
        offset = i - 12;
        i = 12;
      }
      if( i > 0 )
      {
        hd44780_puts( 0, 16 - i, &current_string[offset], i );
        hd44780_puts( 1, 0, &current_string[offset + i], 16 );
      }
      else if( string_wptr )
        hd44780_puts( 1, 16 - string_wptr, current_string, string_wptr );
    }
  }
}

void
tx_cw_thread( void const *argument )
{
  UNUSED( argument );
  u32           notify;
  enum dit_dah  dit_or_dah = SENDING_NOTHING;
  enum paddle_states prev_paddle_state = PAD_NONE;
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( notify & NOTIFY_LETTER )
    {
      pinSet( Tx_PTT_GPIO_Port, Tx_PTT_Pin );
      osDelay( Options.ptt_lead_time );
      while( cw_rptr != cw_wptr )
      {
        send_cw( cw_tx_buffer[cw_rptr++] );
        cw_rptr %= CW_BUF_LEN;
      }
      osDelay( Options.ptt_tail_time );
      pinReset( Tx_PTT_GPIO_Port, Tx_PTT_Pin );
    }
    if( notify & NOTIFY_PADDLE )
    {
      do
      {
        paddle_state =
            pinRead( PAD_L_GPIO_Port,
            PAD_L_Pin ) << ( Options.paddle_mode ==
            PADDLE_NORMAL ) | pinRead( PAD_R_GPIO_Port,
            PAD_R_Pin ) << ( Options.paddle_mode == PADDLE_REVERSE );
        if( Options.keyer_mode == STRAIGHT && paddle_state == PAD_R )
          paddle_state = PAD_NONE;
        if( prev_paddle_state == PAD_NONE && paddle_state != PAD_NONE )
        {
          pinSet( Tx_PTT_GPIO_Port, Tx_PTT_Pin );
          osDelay( Options.ptt_lead_time );
        }
        prev_paddle_state = paddle_state;
        switch ( paddle_state )
        {
          case PAD_L:
            switch ( Options.keyer_mode )
            {
              case STRAIGHT:
                if( dit_or_dah != SENDING_CW )
                {
                  dit_or_dah = SENDING_CW;
                  cw_on(  );
                }
                osDelay( 10 );
                break;
              default:
                dit_or_dah = SENDING_DIT;
                send_dit(  );
                break;
            }
            break;
          case PAD_R:
            switch ( Options.keyer_mode )
            {
              case BUG:
                if( dit_or_dah != SENDING_CW )
                {
                  dit_or_dah = SENDING_CW;
                  cw_on(  );
                }
                osDelay( 10 );
                break;
              default:
                dit_or_dah = SENDING_DAH;
                send_dah(  );
                break;
            }
            break;
          case PAD_LR:
            switch ( Options.keyer_mode )
            {
              case IAMBIC_A:
              case IAMBIC_B:
                if( dit_or_dah & SENDING_DIT )
                {
                  dit_or_dah = SENDING_DAH | SEND_REPEAT;
                  send_dah(  );
                }
                else
                {
                  dit_or_dah = SENDING_DIT | SEND_REPEAT;
                  send_dit(  );
                }
                break;
              case ULTIMATIC:
                if( dit_or_dah & SENDING_DIT )
                {
                  dit_or_dah = SENDING_DIT | SEND_REPEAT;
                  send_dah(  );
                }
                else
                {
                  dit_or_dah = SENDING_DAH | SEND_REPEAT;
                  send_dit(  );
                }
                break;
            }
            break;
          default:
            break;
        }
      }
      while( paddle_state != PAD_NONE );
      switch ( Options.keyer_mode )     // PAD_NONE
      {
        case STRAIGHT:
        case BUG:
          if( dit_or_dah == SENDING_CW )
            cw_off(  );
          dit_or_dah = SENDING_NOTHING;
          break;
        case IAMBIC_B:
          if( dit_or_dah & SEND_REPEAT )
          {
            if( dit_or_dah & SENDING_DIT )
            {
              dit_or_dah = SENDING_DAH;
              send_dah(  );
            }
            else
            {
              dit_or_dah = SENDING_DIT;
              send_dit(  );
            }
          }
          break;
      }
      osDelay( Options.ptt_tail_time );
      pinReset( Tx_PTT_GPIO_Port, Tx_PTT_Pin );
      dit_or_dah = SENDING_NOTHING;
      TaskNotify( thread_tx_decode, NOTIFY_DECODE_SYMBOL );
    }
  }
}

void
tx_decode_thread( void const *argument )
{
  UNUSED( argument );
  u32           notify;
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( notify & NOTIFY_DECODE_SYMBOL )
    {
      osDelay( 5 * morse_timing.dit / 2 );
      decode_tx_morse_code(  );
    }
  }
}

void
beacon_thread( void const *argument )
{
  UNUSED( argument );
  int           len, delay_ms;
  char         *str;
  for( ;; )
  {
    if( Options.memory_repeat_time && Options.beacon_memory )
      delay_ms = 100 * Options.memory_repeat_time;
    else
      delay_ms = 3000;
    osDelay( delay_ms );
    if( main_mode != M_KBD_TX_RX )
      continue;
    if( !Options.beacon_memory )
      continue;
    str = Options.string_memory[Options.beacon_memory-1];
    if( str == NULL )
      continue;
    len = strlen( str );
    len = len > ( MAX_STRING_SIZE - 1 ) ? MAX_STRING_SIZE - 1 : len;
    strncpy( current_string, str, len );
    current_string[len++] = ' ';
    str = current_string;
    while( len-- )
    {
      cw_tx_buffer[cw_wptr++] = *str++;
      cw_wptr %= CW_BUF_LEN;
    }
    TaskNotify( thread_lcd, NOTIFY_LETTER );
    TaskNotify( thread_tx_cw, NOTIFY_CHARS );
  }
}

#if 0
const int     K = target_freq * SAMPLE_BUF_LEN / SAMPLING_RATE;
const float   omega = ( 2.0f * PI * K ) / SAMPLE_BUF_LEN;
//  Goertzel algorithm
float
goertzel( void )
{
  float         coeff = 2.0f * arm_cos_f32( omega );
  float         Q0, Q1, Q2;
  float         magnitude2;     // signal level squared
  Q1 = 0;
  Q2 = 0;
  for( int i = 0; i < SAMPLE_BUF_LEN; i++ )
  {
    Q0 = coeff * Q1 - Q2 + mic_src[i];
    Q2 = Q1;
    Q1 = Q0;
  }
  magnitude2 = ( Q1 * Q1 ) + ( Q2 * Q2 ) - Q1 * Q2 * coeff;
  return sqrt( magnitude2 );
}
#endif

// #define DEBUG_DFT

#ifdef ENABLE_CW_DECODE
u32           decoder_mag_buffer[DECODER_BUF_LEN];
u32           decoder_wptr;
u16           decoder_deltas[DELTAS_BUF_LEN];
u32           deltas_wptr;
float         blackman_window[SAMPLE_BUF_LEN];
float         mic_src[2 * SAMPLE_BUF_LEN];
float         mic_dft[2 * SAMPLE_BUF_LEN];
u64           mag_sum;
u32           mag_avg;
#ifdef DEBUG_DFT
u32           tone_freq, tone_bins, tone_cnt;
u32           decoder_bit_buffer[DECODER_BUF_LEN / 32];
u32           decoder_idx_buffer[DECODER_BUF_LEN / 8];
#endif                          // DEBUG_DFT

void
rx_decode_thread( void const *argument )
{
  UNUSED( argument );
  u32           notify;
  static u8     prev_bits;
  u16           delta;
  arm_rfft_fast_instance_f32 DFT;
  u32          *int_src;
  int           i, int_sample;
  u32           maxmag, maxindex;
#ifdef DEBUG_DFT
  u32           b_index, shift;
#endif                          // DEBUG_DFT
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( notify & NOTIFY_START_CAPTURE )
    {
      if( Options.decoder_mode == DECODER_OFF )
        continue;
      arm_rfft_fast_init_f32( &DFT, 2 * SAMPLE_BUF_LEN );
      // calculate the coefficients for a Blackman window
      for( i = 0; i < SAMPLE_BUF_LEN; i++ )
      {
        blackman_window[i] =
            ( 0.426591f -
            0.496561f * arm_cos_f32( ( 2.0f * PI * i ) /
            SAMPLE_BUF_LEN ) +
            0.076848f * arm_cos_f32( ( 4.0f * PI * i ) / SAMPLE_BUF_LEN ) );
      }
      // start microphone capture into a double buffer
      HAL_I2S_Receive_DblBuf_DMA( &hi2s2,
          ( u16 * ) mic_samples[0], ( u16 * ) mic_samples[1],
          2 * SAMPLE_BUF_LEN );
    }
    if( notify & ( NOTIFY_DFT_BUF0 | NOTIFY_DFT_BUF1 ) )
    {
#ifdef DEBUG_DFT
      pinSet( Tx_GPIO_Port, Tx_Pin );
#endif                          // DEBUG_DFT
      int_src =
          ( notify & NOTIFY_DFT_BUF0 ) ? mic_samples[0] : mic_samples[1];
      for( i = 0; i < SAMPLE_BUF_LEN; i++ )
      {
        int_sample = ( *int_src >> 16 ) | ( *int_src << 16 );
        int_sample >>= 8;       // only 24bits are significant
        int_src += 2;           // skip the right I2S channel which is zero
        // unfortunatelly we can't reuse the samples buffer so we need to copy it
        mic_src[2 * i] = blackman_window[i] * ( float ) int_sample;
        mic_src[2 * i + 1] = 0.0f;
      }
      // calculate the signal DFT
      arm_rfft_fast_f32( &DFT, mic_src, mic_dft );
      mic_dft[1] = 0.0f;        // remove Nyquist value and restore DC as complex
      // search only in a 2KHz bandwidth
      for( maxmag = maxindex = i = 0; i < 8; i++ )
      {
        float         real, imag;
        u32           magnitude2;
        real = mic_dft[2 * i];
        imag = mic_dft[2 * i + 1];
        // scale down and convert to int
        magnitude2 = ( real * real + imag * imag ) / 4096.0f;
        if( magnitude2 > maxmag )
        {
          maxmag = magnitude2;
          maxindex = i;
        }
      }
      mag_sum -= decoder_mag_buffer[decoder_wptr];      // remove oldest
      mag_sum += maxmag;        // update sum
      mag_avg = mag_sum / DECODER_BUF_LEN;      // update average
      decoder_mag_buffer[decoder_wptr] = maxmag;
      prev_bits <<= 1;
      prev_bits |= ( maxmag > mag_avg );        // current bit
      switch ( prev_bits )      // ignore transistions faster than 75 WPM
      {
        default:               // don't do anything
          break;
        case 0b00001110:      // remove 3 high glitches
        case 0b11110001:      // remove 3 low glitches
          delta = decoder_deltas[deltas_wptr] + 4;
          // test for 15 bit overflow
          if( !( ( delta ^ decoder_deltas[deltas_wptr] ) & 1 << 15 ) )
            decoder_deltas[deltas_wptr] = delta;
          prev_bits = ( prev_bits & 0x80 ) ? 0xff : 0;
          break;
        case 0b00000110:      // remove
          // 2
          // high
          // glitches
        case 0b11111001:      // remove 2 low glitches
          delta = decoder_deltas[deltas_wptr] + 3;
          // test for 15 bit overflow
          if( !( ( delta ^ decoder_deltas[deltas_wptr] ) & 1 << 15 ) )
            decoder_deltas[deltas_wptr] = delta;
          prev_bits = ( prev_bits & 0x80 ) ? 0xff : 0;
          break;
        case 0b00000010:      // remove
          // 1
          // high
          // glitch
        case 0b11111101:      // remove 1 low glitch
          delta = decoder_deltas[deltas_wptr] + 2;
          // test for 15 bit overflow
          if( !( ( delta ^ decoder_deltas[deltas_wptr] ) & 1 << 15 ) )
            decoder_deltas[deltas_wptr] = delta;
          prev_bits = ( prev_bits & 0x80 ) ? 0xff : 0;
          break;
        case 0b11100000:      // unchanged
          // 0
        case 0b11000000:
        case 0b10000000:
        case 0b00000000:
        case 0b00011111:      // unchanged 1
        case 0b00111111:
        case 0b01111111:
        case 0b11111111:
          delta = decoder_deltas[deltas_wptr] + 1;
          // test for 15 bit overflow
          if( !( ( delta ^ decoder_deltas[deltas_wptr] ) & 1 << 15 ) )
            decoder_deltas[deltas_wptr] = delta;
          break;
        case 0b00001111:      // transitions
        case 0b11110000:
          deltas_wptr = ( deltas_wptr + 1 ) % DELTAS_BUF_LEN;
          decoder_deltas[deltas_wptr] = ( ( prev_bits & 1 ) << 15 ) | 4;
          break;
      }
      // notify decoding at tone start or after 300ms of silence
      if( decoder_deltas[deltas_wptr] == 0x8004
          || decoder_deltas[deltas_wptr] == 300 / 4 )
        TaskNotify( thread_timing_decode, NOTIFY_NEW_DELTA );
#ifdef DEBUG_DFT
      shift = 1 << ( decoder_wptr % 32 );
      if( c_bit )
      {
        decoder_bit_buffer[decoder_wptr / 32] |= shift;
      }
      else
      {
        decoder_bit_buffer[decoder_wptr / 32] &= ~shift;
        maxindex = 0;
      }
      shift = 4 * ( decoder_wptr % 8 );
      b_index = ( decoder_idx_buffer[decoder_wptr / 8] >> shift ) & 0xf;
      if( b_index )
      {
        tone_bins -= b_index;
        tone_cnt--;
      }
      if( maxindex )
      {
        tone_bins += maxindex;
        tone_cnt++;
      }
      if( tone_cnt )
        tone_freq = 125 + tone_freq * 250 / tone_cnt;
      decoder_idx_buffer[decoder_wptr / 8] &= ~( 0xf << shift );
      decoder_idx_buffer[decoder_wptr / 8] |= maxindex << shift;
      if( decoder_wptr == 0 )
      {
        FILE         *f;
        HAL_I2S_DMAStop( &hi2s2 );
        HAL_I2S_DMAStop( &hi2s3 );
        pinReset( Tx_CW_GPIO_Port, Tx_CW_Pin );
        pinReset( Tx_PTT_GPIO_Port, Tx_PTT_Pin );
        f = debug_fopen( "mag.bin", "wb" );
        debug_fwrite( decoder_mag_buffer, DECODER_BUF_LEN, sizeof( u32 ), f );
        debug_fflush( f );
        debug_fclose( f );
        f = debug_fopen( "index.bin", "wb" );
        debug_fwrite( decoder_idx_buffer, DECODER_BUF_LEN / 8,
            sizeof( u32 ), f );
        debug_fflush( f );
        debug_fclose( f );
        f = debug_fopen( "bits.bin", "wb" );
        debug_fwrite( decoder_bit_buffer, DECODER_BUF_LEN / 32,
            sizeof( u32 ), f );
        debug_fflush( f );
        debug_fclose( f );
        debug_abort(  );
      }
      pinReset( Tx_GPIO_Port, Tx_Pin );
#endif                          // DEBUG_DFT
      decoder_wptr = ( decoder_wptr + 1 ) % DECODER_BUF_LEN;
    }
  }
}

struct delta
{
  u16           delta;
  u16           count;
};

static int
match_ratios( struct delta *d, int len, int max_idx, int *dit, int *dah )
{
  u16           max_delta = ( d + max_idx )->delta;
  u16           max_count = ( d + max_idx )->count;
  u16           limit_count, l_delta, h_delta;
  int           i, l_idx, h_idx;

  limit_count = max_count / 4;
  do
  {
    for( l_idx = -1, i = 0; i < max_idx; i++ )
    {
      if( max_delta / 4 <= ( d + i )->delta &&
          ( d + i )->delta <= max_delta / 2 &&
          ( d + i )->count >= limit_count )
      {
        l_idx = i;
        l_delta = ( d + l_idx )->delta;
      }
    }
    for( h_idx = -1, i = max_idx + 1; i <= len; i++ )
    {
      if( max_delta * 2 <= ( d + i )->delta &&
          ( d + i )->delta <= max_delta * 4 &&
          ( d + i )->count >= limit_count )
      {
        h_idx = i;
        h_delta = ( d + h_idx )->delta;
      }
    }
    limit_count--;
    if( limit_count <= 3 )
      break;
  }
  while( l_idx == -1 && h_idx == -1 );
  if( l_idx >= 0 && h_idx == -1 )
  {
    *dit = l_delta;
    *dah = max_delta;
    return 1;
  }
  if( l_idx == -1 && h_idx >= 0 )
  {
    *dit = max_delta;
    *dah = h_delta;
    return 1;
  }
  return 0;
}

static int
compar_u16( const void *a, const void *b )
{
  if( *( u16 * ) a == *( u16 * ) b )
    return 0;
  if( *( u16 * ) a > *( u16 * ) b )
    return 1;
  return -1;
}

static int
get_dit_dah_timing( struct dit_dah_timing *tm )
{
  static struct delta quiet[DELTAS_BUF_LEN / 8], tone[DELTAS_BUF_LEN / 8];
  static u16    sorted_deltas[DELTAS_BUF_LEN];
  u16          *v;
  u16           quiet_max_cnt, tone_max_cnt;
  int           quiet_max_idx, tone_max_idx;
  int           i, dit, dah, quiet_ptr, tone_ptr;
  memcpy( sorted_deltas, decoder_deltas, DELTAS_BUF_LEN * sizeof( u16 ) );
  if( sorted_deltas[deltas_wptr] == 0x8004 )
    sorted_deltas[deltas_wptr] = 0;
  qsort( sorted_deltas, DELTAS_BUF_LEN, sizeof( u16 ), compar_u16 );
  memset( quiet, 0, sizeof( quiet ) );
  memset( tone, 0, sizeof( tone ) );
  quiet_max_idx = tone_max_idx = -1;
  quiet_max_cnt = tone_max_cnt = 0;
  quiet_ptr = tone_ptr = 0;
  for( v = sorted_deltas, i = 0; i < DELTAS_BUF_LEN; i++, v++ )
  {
    if( ( *v & ~0x8000 ) == 0 )
      continue;
    if( *v & 0x8000 )
    {
      if( tone_ptr < DELTAS_BUF_LEN / 8 )
      {
        *v &= ~0x8000;
        if( tone[tone_ptr].delta != *v )
        {
          if( tone[tone_ptr].delta )
          {
            tone_ptr++;
            if( tone_ptr == DELTAS_BUF_LEN / 8 )
              continue;
          }
          tone[tone_ptr].delta = *v;
          tone[tone_ptr].count = 1;
        }
        else
        {
          tone[tone_ptr].count++;
        }
        if( tone[tone_ptr].count > tone_max_cnt )
        {
          tone_max_cnt = tone[tone_ptr].count;
          tone_max_idx = tone_ptr;
        }
      }
    }
    else
    {
      if( quiet_ptr < DELTAS_BUF_LEN / 8 )
      {
        if( quiet[quiet_ptr].delta != *v )
        {
          if( quiet[quiet_ptr].delta )
          {
            quiet_ptr++;
            if( quiet_ptr == DELTAS_BUF_LEN / 8 )
              continue;
          }
          quiet[quiet_ptr].delta = *v;
          quiet[quiet_ptr].count = 1;
        }
        else
        {
          quiet[quiet_ptr].count++;
        }
        if( quiet[quiet_ptr].count > quiet_max_cnt )
        {
          quiet_max_cnt = quiet[quiet_ptr].count;
          quiet_max_idx = quiet_ptr;
        }
      }
    }
  }
  if( !tone_ptr || !quiet_ptr )
    return 0;
  if( !match_ratios( tone, tone_ptr, tone_max_idx, &tm->dit, &tm->dah ) )
    return 0;
  // manual operator (no fixed interchar/interword timing)
  if( quiet_ptr == DELTAS_BUF_LEN / 8
      || !match_ratios( quiet, quiet_ptr, quiet_max_idx, &dit, &dah ) )
  {
    tm->intercharacter = 3 * tm->dit;
    tm->interword = 30 * tm->dit;
    return 1;
  }
  tm->dit = ( tm->dit + dit ) / 2;      // average
  tm->intercharacter = ( tm->dah + dah ) / 2;   // average
  tm->interword = ( 21 * tm->dit + 7 * tm->intercharacter ) / 6;        // average
  return 1;
}

char          decoder_char_buffer[CHAR_BUF_LEN];

void
timing_decode_thread( void const *argument )
{
  UNUSED( argument );
  u32           notify;
  u32           symbol;
  u32           deltas_rptr;
  bool          tone;
  int           i;
  u16           delta;
  struct dit_dah_timing timing;
  for( ;; )
  {
    xTaskNotifyWait( 0, -1, &notify, portMAX_DELAY );
    if( notify & NOTIFY_NEW_DELTA )
    {
      if( !get_dit_dah_timing( &timing ) )
        continue;
      deltas_rptr = deltas_wptr;
      if( decoder_deltas[deltas_rptr] & 0x8000 )
        deltas_rptr = ( deltas_rptr - 1 ) % DELTAS_BUF_LEN;
      for( i = CHAR_BUF_LEN - 1; i >= 0; )
      {
        symbol = 0;
        while( 1 )
        {
          tone = !!( decoder_deltas[deltas_rptr] & 0x8000 );
          delta = decoder_deltas[deltas_rptr] & ~0x8000;
          deltas_rptr = ( deltas_rptr - 1 ) % DELTAS_BUF_LEN;
          if( deltas_rptr == deltas_wptr )
            break;
          if( tone )
          {
            symbol <<= 4;
            // longer than 1.75 dit is a dah
            if( delta > 7 * timing.dit / 4 )
              symbol |= 3;
            else
              symbol |= 1;
          }
          else                  // silence
          {
            if( delta >= 2 * timing.dit ||
                delta >= 2 * timing.intercharacter / 3 )
              break;
          }
        }
        if( deltas_rptr == deltas_wptr )
          break;
        if( symbol )
          decoder_char_buffer[i--] = decode_symbol( symbol );
        if( delta >= 6 * timing.interword / 7 && i != CHAR_BUF_LEN - 1 )
          decoder_char_buffer[i--] = ' ';
      }
      for( ; i >= 0; )
        decoder_char_buffer[i--] = ' ';
    }
    TaskNotify( thread_lcd, NOTIFY_DECODER_LINE );
  }
}
#endif                          // ENABLE_CW_DECODE
