/* ----------------------------------------------------------------------
 * Project:	 CMSIS DSP Library
 * Title:	 arm_cfft_init_f32.c
 * Description:	 Initialization function for cfft f32 instance
 *
 * $Date:	 23 April 2021
 * $Revision:	 V1.9.0
 *
 * Target Processor: Cortex-M and Cortex-A cores
 * -------------------------------------------------------------------- */

#define FFTINIT(EXT,SIZE)					    \
  S->bitRevLength = arm_cfft_sR_##EXT##_len##SIZE.bitRevLength;	       \
  S->pBitRevTable = arm_cfft_sR_##EXT##_len##SIZE.pBitRevTable;		\
  S->pTwiddle = arm_cfft_sR_##EXT##_len##SIZE.pTwiddle;

/**
  @addtogroup ComplexFFT
  @{
 */

/**
  @brief	 Initialization function for the cfft f32 function
  @param[in,out] S		points to an instance of the floating-point CFFT structure
  @param[in]	 fftLen		fft length (number of complex samples)
  @return	 execution status
		   - \ref ARM_MATH_SUCCESS	  : Operation successful
		   - \ref ARM_MATH_ARGUMENT_ERROR : an error is detected

  @par		Use of this function is mandatory only for the MVE version of the FFT.
		Other versions can still initialize directly the data structure using
		variables declared in arm_const_structs.h
 */

#include "transform_functions.h"
#include "arm_common_tables.h"
#include "arm_const_structs.h"

arm_status
arm_cfft_init_f32( arm_cfft_instance_f32 * S, uint16_t fftLen )
{
  /* Initialise the default arm status */
  arm_status    status = ARM_MATH_SUCCESS;

  /* Initialise the FFT length */
  S->fftLen = fftLen;

  /* Initialise the Twiddle coefficient pointer */
  S->pTwiddle = NULL;

  /* Initializations of Instance structure depending on the FFT length */
  switch ( S->fftLen )
  {
#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_64) && defined(ARM_TABLE_BITREVIDX_FLT_64))
    case 64U:
      FFTINIT( f32, 64 );
      break;
#endif
    default:
      /* Reporting argument error if fftSize is not valid value */
      status = ARM_MATH_ARGUMENT_ERROR;
      break;
  }

  return ( status );
}
