/******************************************************************************
 * @file     transform_functions.h
 * @brief    Public header file for CMSIS DSP Library
 * @version  V1.9.0
 * @date     23 April 2021
 * Target Processor: Cortex-M and Cortex-A cores
 ******************************************************************************/

#ifndef _TRANSFORM_FUNCTIONS_H_
#define _TRANSFORM_FUNCTIONS_H_

#include "arm_math_types.h"

#include "none.h"
#include "utils.h"

#include "basic_math_functions.h"
#include "complex_math_functions.h"

#ifdef	 __cplusplus
extern "C"
{
#endif

/**
 * @defgroup groupTransforms Transform Functions
 */

  /**
   * @brief Instance structure for the floating-point CFFT/CIFFT function.
   */
  typedef struct
  {
	  uint16_t fftLen;		     /**< length of the FFT. */
    const float32_t *pTwiddle;		     /**< points to the Twiddle factor table. */
    const uint16_t *pBitRevTable;	     /**< points to the bit reversal table. */
	  uint16_t twidCoefModifier;	     /**< twiddle coefficient modifier that supports different size FFTs with the same twiddle factor table. */
	  uint16_t bitRevFactor;	     /**< bit reversal modifier that supports different size FFTs with the same bit reversal table. */
	  float32_t onebyfftLen;	     /**< value of 1/fftLen. */
  } arm_cfft_radix2_instance_f32;

  /**
   * @brief Instance structure for the floating-point CFFT/CIFFT function.
   */
  typedef struct
  {
	  uint16_t fftLen;		     /**< length of the FFT. */
    const float32_t *pTwiddle;		     /**< points to the Twiddle factor table. */
    const uint16_t *pBitRevTable;	     /**< points to the bit reversal table. */
	  uint16_t twidCoefModifier;	     /**< twiddle coefficient modifier that supports different size FFTs with the same twiddle factor table. */
	  uint16_t bitRevFactor;	     /**< bit reversal modifier that supports different size FFTs with the same bit reversal table. */
	  float32_t onebyfftLen;	     /**< value of 1/fftLen. */
  } arm_cfft_radix4_instance_f32;

  /**
   * @brief Instance structure for the floating-point CFFT/CIFFT function.
   */
  typedef struct
  {
	  uint16_t fftLen;		     /**< length of the FFT. */
    const float32_t *pTwiddle;	       /**< points to the Twiddle factor table. */
    const uint16_t *pBitRevTable;      /**< points to the bit reversal table. */
	  uint16_t bitRevLength;	     /**< bit reversal table length. */
  } arm_cfft_instance_f32;

  arm_status arm_cfft_init_f32(
  arm_cfft_instance_f32 * S,
  uint16_t fftLen);

  void arm_cfft_f32(
  const arm_cfft_instance_f32 * S,
	float32_t * p1 );

  /**
   * @brief Instance structure for the floating-point RFFT/RIFFT function.
   */
  typedef struct
  {
	  uint32_t fftLenReal;			      /**< length of the real FFT. */
	  uint16_t fftLenBy2;			      /**< length of the complex FFT. */
	  uint32_t twidCoefRModifier;			  /**< twiddle coefficient modifier that supports different size FFTs with the same twiddle factor table. */
    const float32_t *pTwiddleAReal;		      /**< points to the real twiddle factor table. */
    const float32_t *pTwiddleBReal;		      /**< points to the imag twiddle factor table. */
	  arm_cfft_radix4_instance_f32 *pCfft;	      /**< points to the complex FFT instance. */
  } arm_rfft_instance_f32;

  arm_status arm_rfft_init_f32(
	arm_rfft_instance_f32 * S,
	arm_cfft_radix4_instance_f32 * S_CFFT,
	uint32_t fftLenReal );

  void arm_rfft_f32(
  const arm_rfft_instance_f32 * S,
	float32_t * pSrc,
	float32_t * pDst);

  /**
   * @brief Instance structure for the floating-point RFFT/RIFFT function.
   */
typedef struct
  {
	  arm_cfft_instance_f32 Sint;	   /**< Internal CFFT structure. */
	  uint16_t fftLenRFFT;		   /**< length of the real sequence */
    const float32_t * pTwiddleRFFT;	   /**< Twiddle factors real stage  */
  } arm_rfft_fast_instance_f32 ;

arm_status arm_rfft_fast_init_f32 (
	 arm_rfft_fast_instance_f32 * S,
	 uint16_t fftLen);

  void arm_rfft_fast_f32(
	const arm_rfft_fast_instance_f32 * S,
	float32_t * p, float32_t * pOut );

#endif /* ifndef _TRANSFORM_FUNCTIONS_H_ */
